# Copyright (c) 2013 Santosh Philip
# =======================================================================
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
# =======================================================================

"""get data from weather.stat file"""

def getlocation(fhandle):
    """get the location from the weather stat file"""
    for line in fhandle:
        line = line.strip()
        if line.startswith('Location'):
            location = line.split('--')[-1]
            location = location.strip()
            break
    line = fhandle.next()
    lat, lng, tz = locationdatastrings(line)
    lat = longlat(lat)
    lng = longlat(lng)
    tz = timezone(tz)
    return location, lat, lng, tz
    
            
def longlat(txt):
    """convert N 33° 30' into +33.5
    W 33° 30' into -33.5
    expects the format to be exactly N 33° 30' """
    txt = txt.strip()
    words = txt.split()
    deg = words[1].split('°')[0]
    deg = float(deg)
    try:
        minutes = words[2].split("'")[0]
        minutes = float(minutes)/60.
    except IndexError, e:
        minutes = 0
    angle = deg + minutes
    cardinal = words[0].strip()
    if cardinal == 'S' or cardinal == 'W':
        angle = angle * -1.
    return angle
    
def timezone(txt):
    """from GMT -7.0 Hours return -7.0"""
    txt = txt.strip()
    words = txt.split()
    return float(words[1])
    
def locationdatastrings(txt):
    """return the location data string snippets: lat, lonlngg, tz"""
    txt = txt.strip()
    words = txt.split('}')
    lat = words[0]
    lat = lat.split('{')[-1]
    lng = words[1]
    lng = lng.split('{')[-1]
    tz = words[2]
    tz = tz.split('{')[-1]
    return lat, lng, tz
    
    


"""helpers for pytest"""
# taken from python's unit test
# may be covered by Python's license

from __future__ import print_function


def almostequal(first, second, places=7, verbose=True):
    # """Tests if two numbers are almost equal
    #
    # Used during unit testing with pytest
    #
    # """
    """Tests if two numbers are almost equal

    Used during unit testing with pytest

    Parameters
    ----------
    first : float/int
        first number can be a float or an int
    second : float/int
        second number can be a float or an int
    places : int
        if places is 1, it will check equality up to the first decimal place.
        Default places=7
    verbose : bool
        Will print details if first and second are not equal

    Returns
    -------
    bool
        Returns True it first and second are equal

    """
    if round(abs(second-first), places) != 0:
        if verbose:
            print(round(abs(second-first), places))
            print("notalmost: %s != %s" % (first, second))
        return False
    else:
        return True



# Copyright (c) 2013 Santosh Philip
# =======================================================================
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
# =======================================================================

"""py.test for weather_stat"""
import weather_stats
from StringIO import StringIO

# getlocation
def test_getlocation():
    """py.test for getlocation"""
    data = ((""" Statistics for USA_AZ_Phoenix.722780_TMY2
 Location -- PHOENIX AZ USA  
      # {N 33° 30'} {W 112°  30'} {GMT -7.0 Hours}
""", ("PHOENIX AZ USA", 33.5, -112.5, -7.0)), # stattxt, location
    )
    for stattxt, location in data:
        fhandle = StringIO(stattxt)
        result = weather_stats.getlocation(fhandle)
        assert result == location

def test_longlat():
    """py.test for longlat"""
    import sys
    reload(sys)
    sys.setdefaultencoding("utf-8")
    # data = (("""N 33\xc2\xb0 25'""", 33.5), # txt, ang
    data = (("""N 33° 30'""", 33.5), # txt, ang
    ("""E 33° 30'""", 33.5), # txt, ang
    ("""S 33° 30'""", -33.5), # txt, ang
    ("""W 33° 30'""", -33.5), # txt, ang
    ("""E 33° """, 33.0), # txt, ang
    )
    for txt, ang in data:
        result = weather_stats.longlat(txt)
        assert result == ang
        
def test_timezone():
    """py.test for timezone"""
    data = (("GMT -7.0 Hours", -7.0), # txt, tz
    )
    for txt, tz in data:
        result = weather_stats.timezone(txt)
        assert result == tz
        
def test_locationdatastrings():
    """py.test locationdatastrings"""
    data = (("      {N 33° 25'} {W 112°  1'} {GMT -7.0 Hours}",
        "N 33° 25'", "W 112°  1'", "GMT -7.0 Hours"), # txt, lat, lng, tz
    )
    for txt, lat, lng, tz in data:
        result = weather_stats.locationdatastrings(txt)
        assert result == (lat, lng, tz)


# Copyright (c) 2013 Santosh Philip
# =======================================================================
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
# =======================================================================

"""py.test for clearskyrad.py"""
from io import StringIO   
from pyclearsky import clearskyrad
from pyclearsky.pytest_helpers import almostequal
from datetime import datetime


def test_func():
    """py.test for func"""
    result = clearskyrad.func(1, 2)
    assert result


def test_degcos():
    """py.test for degcos"""
    data = ((60, 0.5),  # deg, thecos
            )
    for deg, thecos in data:
        result = clearskyrad.degcos(deg)
        assert almostequal(result, thecos) is True


def test_degsin():
    """py.test for degcos"""
    data = ((30, 0.5),  # deg, thesin
            )
    for deg, thesin in data:
        result = clearskyrad.degsin(deg)
        assert almostequal(result, thesin) is True


def test_ETradiation():
    """py.test for ETradiation"""
    data = (
        (21, 1410),  # daynum, radiation
        (52, 1397),  # daynum, radiation
        (80, 1378),  # daynum, radiation
        (111, 1354),  # daynum, radiation
        (141, 1334),  # daynum, radiation
        (172, 1323),  # daynum, radiation
        (202, 1324),  # daynum, radiation
        (233, 1336),  # daynum, radiation
        (264, 1357),  # daynum, radiation
        (294, 1380),  # daynum, radiation
        (325, 1400),  # daynum, radiation
        (355, 1411),  # daynum, radiation
    )
    for daynum, radiation in data:
        result = clearskyrad.ETradiation(daynum=daynum)
        assert almostequal(result, radiation, places=0) is True
    from datetime import datetime as dt
    data = (
        (dt(2013, 1, 21), 1410),  # thedate, radiation
        (dt(2013, 2, 21), 1397),  # thedate, radiation
        (dt(2013, 3, 21), 1378),  # thedate, radiation
        (dt(2013, 4, 21), 1354),  # thedate, radiations
        (dt(2013, 5, 21), 1334),  # thedate, radiation
        (dt(2013, 6, 21), 1323),  # thedate, radiation
        (dt(2013, 7, 21), 1324),  # thedate, radiation
        (dt(2013, 8, 21), 1336),  # thedate, radiation
        (dt(2013, 9, 21), 1357),  # thedate, radiation
        (dt(2013, 10, 21), 1380),  # thedate, radiation
        (dt(2013, 11, 21), 1400),  # thedate, radiation
        (dt(2013, 12, 21), 1411),  # thedate, radiation
    )
    for thedate, radiation in data:
        result = clearskyrad.ETradiation(thedate=thedate)
        # print result, radiation
        assert almostequal(result, radiation, places=0) is True


def test_airmass():
    """py.test for airmass"""
    data = (
        (30, 1.9942928525),  # alt, theairmass
        (45, 1.412595252),  # alt, theairmass
        (60, 1.1539922334),  # alt, theairmass
        (90, 0.9997119919),  # alt, theairmass
    )
    for alt, theairmass in data:
        result = clearskyrad.airmass(alt)
        assert almostequal(result, theairmass)


def test_tau():
    """py.test for tau"""
    data = (
    (""" - Displaying Monthly Design Conditions "Climate Design Data 2009 ASHRAE Handbook"
 - Monthly Optical Sky Depth Beam (taub) and Diffuse (taud)
 	                        	Jan	Feb	Mar	Apr	May	Jun	Jul	Aug	Sep	Oct	Nov	Dec
 	             taub (beam)	0.289	 0.29	0.325	0.351	0.377	 0.37	0.362	0.352	0.343	0.323	0.302	0.289
 	          taud (diffuse)	2.641	2.681	2.392	2.292	2.224	2.361	2.489	2.561	2.539	2.534	2.618	2.633

 	                    taub	= Clear Sky Optical Depth for Beam Irradiance
 	                    taud	= Clear Sky Optical Depth for Diffuse Irradiance

""",  # noqa: E112, E501, W191, E101 -> copied from weather file and triggering these flake8 warnings
        [0.289, 0.29, 0.325, 0.351, 0.377, 0.37,
         0.362, 0.352, 0.343, 0.323, 0.302, 0.289],
        [2.641, 2.681, 2.392, 2.292, 2.224, 2.361,
         2.489, 2.561, 2.539, 2.534, 2.618, 2.633]
    ),  # txt, taub, taud
        )
    for txt, taub, taud in data:
        fhandle = StringIO(txt)
        result = clearskyrad.tau(fhandle)
        assert result == (taub, taud)


def test_getab():
    """py.test for getab"""
    data = (
        (0.289, 2.641, 0.652079204),  # taub, taud, theab
    )
    for taub, taud, theab in data:
        result = clearskyrad.getab(taub, taud)
        assert almostequal(result, theab)


def test_getad():
    """py.test for getad"""
    data = (
        # (0.289, 2.641, -0.335194893),  # taub, taud, thead
        (0.556, 1.779, 0.310),  # taub, taud, thead
    )
    for taub, taud, thead in data:
        result = clearskyrad.getad(taub, taud)
        assert almostequal(result, thead, 3)


def test_directnormal_inner():
    """py.test for directnormal_inner"""
    data = (
        (1409.962705, 0.289, 0.999711992,
         0.652079204, 1056.136599),  # E0, taub,  m, ab, Eb
    )
    for E0, taub,  m, ab, Eb in data:
        result = clearskyrad.directnormal_inner(E0, taub, m, ab)
        assert almostequal(result, Eb, places=6)


def test_diffhoriz_inner():
    """py.test for diffhoriz_inner"""
    data = (
        (1409.962705, 2.641, 0.999711992,
         -0.335194893, 100.490533,),  # E0, taud, m, ad, Ed
    )
    for E0, taud, m, ad, Ed in data:
        result = clearskyrad.diffhoriz_inner(E0, taud, m, ad)
        assert almostequal(result, Ed,  places=6)


def test_directnormal():
    """py.test for directnormal"""
    data = (
        (0.289, 2.641, 90, 21, 1056.136599),  # taub, taud, alt, daynum, Eb
    )
    for taub, taud, alt, daynum, Eb in data:
        result = clearskyrad.directnormal(taub, taud, alt, daynum)
        assert almostequal(result, Eb, places=5)


def test_diffusehorizontal():
    """py.test for diffusehorizontal"""
    data = (
        (0.289, 2.641, 90, 21, 100.528187871),  # taub, taud, alt, daynum, Eb
    )
    for taub, taud, alt, daynum, Eb in data:
        result = clearskyrad.diffusehorizontal(taub, taud, alt, daynum=daynum)
        assert almostequal(result, Eb)


def test_weatherdata():
    """py.test with real weather data"""
    # from USA_AZ_Phoenix.722780_TMY2.ddy
    # datetime = month, date, hour = 1, 24, 13
    # ET Horizontal Radiation = 852
    # ET Direct Normal radiation = 1412
    # Direct Normal Radiation = 969
    # Diffuse horizontal radiation = 71
    # Total Sky cover = 0
    # Opaque sky cover = 0
    # from USA_AZ_Phoenix.722780_TMY2.stat
    # Jan taub = 0.306
    # Jan taud = 2.534
    # Feb taub = 0.317
    # Feb taud = 2.463
    # from <http://www.esrl.noaa.gov/gmd/grad/solcalc/azel.html>
    # solar azimuth = 185.8 degrees
    # solar altitude = 37.36 degrees

    # Eb = 969
    taub = 0.306
    taud = 2.534
    alt = 37.36
    daynum = 24
    result = clearskyrad.directnormal(taub, taud, alt, daynum=daynum)
    # assert result == Eb

    Ed = 71
    result = clearskyrad.diffusehorizontal(taub, taud, alt, daynum=daynum)
    # assert result == Ed

    # other days
    # 1975, 2, 16, 14
    # Eb = 816
    taub = 0.317
    taud = 2.463
    alt = 40.67
    daynum = 24
    thedate = datetime(1975, 2, 16)
    result = clearskyrad.directnormal(taub, taud, alt, thedate=thedate)
    # assert result == Eb

    Ed = 98.3801828381502
    result = clearskyrad.diffusehorizontal(taub, taud, alt, thedate=thedate)
    assert result == Ed



# Copyright (c) 2013 Santosh Philip
# =======================================================================
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
# =======================================================================

"""test ashraesky against weather file
compare outputs against weather data from clear sky days."""

import sys
# sys.path.append('/Users/santosh/Documents/coolshadow/sunpath/spapyrex')
sys.path.append('/Users/oompag/Documents/coolshadow/sunpath/spapyrex')

import spabase
import calendar
import clearskyrad
from datetime import datetime

# phoenix Arizona
spadict = {}
spadict['year'] = 1975 
spadict['month'] = 2
spadict['day'] = 16
spadict['hour'] = 14
spadict['minute'] = 00
spadict['second'] = 00
spadict['timezone'] = -7.0 # negative west of Greenwich
                           # -12   to   12 hours
spadict['delta_t'] = 67 # Difference between earth rotation time 
                        # and terrestrial time
spadict['longitude'] = -112.066 # negative west of Greenwich
                                # -180  to  180 degrees
spadict['latitude'] = 33.43 # N is +ve
spadict['elevation'] = 339.14 # meters
spadict['pressure'] = 820 # millibars
spadict['temperature'] = 11 # C
spadict['slope'] = 0
spadict['azm_rotation'] = 0
spadict['atmos_refract'] = 0.5667 # typical value

statfilename = './weatherfiles/USA_AZ_Phoenix/USA_AZ_Phoenix.722780_TMY2.stat'
taubs, tauds = clearskyrad.tau(open(statfilename))

# for m in range(1, 13):
for m in range(1, 13):
    for d in range(1, calendar.monthrange(spadict['year'], m)[1] + 1):
    # for d in range(1, 25):
        for h in range(1, 25):
            spadict['month'] = m
            spadict['day'] = d
            spadict['hour'] = h
            try:
                sun = spabase.spacalc(spadict)
                alt = 90 - sun['zenith']
                taub, taud = taubs[m -1], tauds[m -1]
                thedate = datetime(spadict['year'], m, d)
                if alt <= 0:
                    Eb = 0
                    Ed = 0
                else:
                    Eb = clearskyrad.directnormal(taub, taud, alt, thedate=thedate)
                    Ed = clearskyrad.diffusehorizontal(taub, taud, alt, thedate=thedate)
                print '%s, %s, %s, %s, %s, %s' % (m, d, h, alt, Eb, Ed)
                # print m, d, h, 90 - sun['zenith']
                # print sun
            except spabase.UnknownError:
                raise spabase.DataError



# Copyright (c) 2013 Santosh Philip
# =======================================================================
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
# =======================================================================

import csv
txt = open('b.csv', 'r').read()
lines = txt.splitlines()
rows = [line.split(',') for line in lines]
rows = [[float(cell) for cell in row] for row in rows]
brows = [[row[4], row[6]] for row in rows if row[-1] == 0 and row[-2] == 0]
csv_writer = csv.writer(open('brows.csv', 'w')).writerows(brows)

drows = [[row[5], row[7]] for row in rows if row[-1] == 0 and row[-2] == 0]
csv_writer = csv.writer(open('drows.csv', 'w')).writerows(drows)














# b0rows = [row for row in brows if row[0] == 0 and row[1] == 0]
# csv_writer = csv.writer(open('brows.csv', 'w')).writerows(b0rows)
# b0rows = [row for row in brows if row[0] != 0 and row[1] != 0]
# csv_writer = csv.writer(open('brows.csv', 'w')).writerows(b0rows)
# csv_writer = csv.writer(open('brows.csv', 'w')).writerows(b0rows)
# b0rows = [row[0] - row[1] for row in brows if row[0] != 0 and row[1] != 0]
# csv_writer = csv.writer(open('brows.csv', 'w')).writerows(b0rows)
# b0rows = [[row[0] - row[1]] for row in brows if row[0] != 0 and row[1] != 0]
# csv_writer = csv.writer(open('brows.csv', 'w')).writerows(b0rows)
# b0rows = [row for row in brows if row[0] != 0 and row[1] != 0]
# csv_writer = csv.writer(open('brows.csv', 'w')).writerows(b0rows)
# hist



# Copyright (c) 2013 Santosh Philip
# =======================================================================
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
# =======================================================================
"""calcs for clearsky radiation from Ashrae fundamentals 2009. page 14.9"""

import math



def degcos(theta):
    """return cos(theta) when theta is in degrees"""
    theta = math.radians(theta)
    return math.cos(theta)
    
def degsin(theta):
    """docstring for degsin"""
    theta = math.radians(theta)
    return math.sin(theta)


def ETradiation(daynum=None, thedate=None):
    """extraterrestrial radiation for specific day
    Equation 4.
    E0 = Esc (1 + 0.033 * cos(360 * (n - 3) / 365))"""
    if daynum:
        pass
    else:
        daynum = thedate.timetuple().tm_yday
    Esc = 1367.
    # print Esc
    # print 360. * (daynum - 3) / 365.
    # print math.radians(360. * (daynum - 3) / 365.)
    # print degcos(360. * (daynum - 3) / 365.)
    E0 = Esc * (1 + 0.033 * degcos(360. * (daynum - 3) / 365.))
    # print E0
    return E0
    
def airmass(alt):
    """return the airmass
    Equation 16
    m = 1 / [sin(b) + 0.50572 * power(6.07995 + b, -1.6364)]"""
    b = alt
    m = 1 / (degsin(b) + 0.50572 * math.pow(6.07995 + b, -1.6364))
    return m
    
def tau(fhandle):
    """return tau_b, tau_d for each month of year"""
    for line in fhandle:
        line = line.strip()
        if line.startswith("taub"):
            words = line.split()
            taub = words[-12:]
            taub = [float(word) for word in taub]
            break
    for line in fhandle:
        line = line.strip()
        if line.startswith("taud"):
            words = line.split()
            taud = words[-12:]
            taud = [float(word) for word in taud]
            break
    return taub, taud
    
def getab(taub, taud):
    """return ab in equation 19
    """
    v1 = 1.219
    v2 = 0.043 * taub
    v3 = 0.151 * taud
    v4 = 0.204 * taub * taud
    return v1 - v2 - v3 - v4
        
def getad(taub, taud):
    """return ad in equation 20
    """
    # print taub, taud
    v1 = 0.202
    v2 = 0.852 * taub
    v3 = 0.007 * taud
    v4 = 0.357 * taub * taud
    # print v1, v2, v3, v4
    # return v1 - v2 - v3 - v4 # typo in the equation. Example is correct. 
    return v1 + v2 - v3 - v4
        

def directnormal_inner(E0, taub, m, ab):
    """return the direct normal radiation
    equation 17
    Eb = E0 * exp( - taub * power(m, ab) )
    """ 
    # Eb args = E0, taub, m, ab
    # E0 = ETradiation(daynum=None, thedate=None)
    # taub
    # m = airmass(alt)
    # ab = ab(taub, taud)
    Eb = E0 * math.exp(-taub * math.pow(m, ab))
    return Eb
    
def diffhoriz_inner(E0, taud, m, ad):
    """return diffuse horizontal radiation
    equation 18
    Ed = E0 * exp(-taud * power(m, ad))"""
    # print math.exp(-taud * math.pow(m, ad))
    Ed = E0 * math.exp(-taud * math.pow(m, ad))
    return Ed
    
def directnormal(taub, taud, alt, daynum=None, thedate=None):
    """return direct normal radiation
    see directnormal_inner for details"""
    E0 = ETradiation(daynum=daynum, thedate=thedate)
    m = airmass(alt)
    ab = getab(taub, taud)
    Eb = directnormal_inner(E0, taub, m, ab)
    return Eb

def diffusehorizontal(taub, taud, alt, daynum=None, thedate=None):
    """return the diffuce horizontal radiation
    see diffhoriz_inner for details"""
    E0 = ETradiation(daynum=daynum, thedate=thedate)
    m = airmass(alt)
    ad = getad(taub, taud)
    # print E0, taud, m, ad
    Ed = diffhoriz_inner(E0, taud, m, ad)
    return Ed


# Copyright (c) 2013, 2018 Santosh Philip
# =======================================================================
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
# =======================================================================
"""calcs for clearsky radiation from Ashrae fundamentals 2009. page 14.9"""

import math


def func(arg1, arg2):
    """Summary line of func.

    Extended description of function. This is a Sample
    Function to show how the function documentation should be done.
    This is the numpy style documentation,
    will be formatted beautifully by sphinx-napoleon extension

    Parameters
    ----------
    arg1 : int
        Description of arg1
    arg2 : str
        Description of arg2

    Returns
    -------
    bool
        Description of return value

    """
    return True


def degcos(theta):
    """return cos(theta) where theta is in degrees

    Parameters
    ----------
    theta : float
        angle in degrees

    Returns
    -------
    float
        cosine value of the angle theta (theta is in degrees)

    """
    theta = math.radians(theta)
    return math.cos(theta)


def degsin(theta):
    """return sin(theta) where theta is in degrees

    Parameters
    ----------
    theta : float
        angle in degrees

    Returns
    -------
    float
        sine value of the angle theta (theta is in degrees)

    """
    theta = math.radians(theta)
    return math.sin(theta)


def ETradiation(daynum=None, thedate=None):
    """Calculates Extraterrestial Radiation for a specific day

    This is equation (4) from
    ASHRAE Fundamentals 2009, page 14.7.

    The equation is:

    - E0 = Esc (1 + 0.033 * cos(360 * (n - 3) / 365))
        - where n = daynum

    ETradiation takes *daynum* OR *thedate* as an input
        - daynum is the day of the year
            - (1 for January 1, 32 for February 1, etc)
        - thedate is a datetime.datetime object
            - ETradiation will calculate the daynum from the date


    Parameters
    ----------
    daynum : int
        daynum is the day of the year
    thedate : datetime.datetime()
        daynum is calculated from the datetime object

    Returns
    -------
    float
        Extraterrestial Radiation for a specific day

"""
    if daynum:
        pass
    else:
        daynum = thedate.timetuple().tm_yday
    Esc = 1367.
    E0 = Esc * (1 + 0.033 * degcos(360. * (daynum - 3) / 365.))
    return E0


def airmass(alt):
    """Calculates the airmass

    This is equation (16) from
    ASHRAE Fundamentals 2009, page 14.9.

    The equation is:

    -  m = 1 / [sin(b) + 0.50572 * power(6.07995 + b, -1.6364)]
        - where b = solar altitude


    Parameters
    ----------
    alt : float
        solar altitude

    Returns
    -------
    float
        Air Mass

"""
    b = alt
    m = 1 / (degsin(b) + 0.50572 * math.pow(6.07995 + b, -1.6364))
    return m


def tau(fhandle):
    """return tau_b and tau_d for each month of the year

    These values are in the weather file with extension .stat

    Parameters
    ----------
    fhandle : filehandle
        fhandle is a file open for reading

    Returns
    -------
    taub : list
    taud : list
        return two lists with values of taub and taud

"""
    for line in fhandle:
        line = line.strip()
        if line.startswith("taub"):
            words = line.split()
            taub = words[-12:]
            taub = [float(word) for word in taub]
            break
    for line in fhandle:
        line = line.strip()
        if line.startswith("taud"):
            words = line.split()
            taud = words[-12:]
            taud = [float(word) for word in taud]
            break
    return taub, taud


def getab(taub, taud):
    """calculates value of ab in equation (19)

    Equation (19) in ASHRAE Fundamentals 2009 pg 14.9

    Parameters
    ----------
    taub : float
        value of taub in equation (19)
    taud : float
        value of taud in equation (19)

    Returns
    -------
    float
        the value of ab

    """
    v1 = 1.219
    v2 = 0.043 * taub
    v3 = 0.151 * taud
    v4 = 0.204 * taub * taud
    return v1 - v2 - v3 - v4


def getad(taub, taud):
    """calculates value of ad in equation (20)

    Equation (20) in ASHRAE Fundamentals 2009 pg 14.9

    Parameters
    ----------
    taub : float
        value of taub in equation (20)
    taud : float
        value of taud in equation (20)

    Returns
    -------
    float
        the value of ad

    """
    # """return ad in equation 20
    # """
    v1 = 0.202
    v2 = 0.852 * taub
    v3 = 0.007 * taud
    v4 = 0.357 * taub * taud
    # print v1, v2, v3, v4
    # return v1 - v2 - v3 - v4 # typo in the equation. Example is correct.
    return v1 + v2 - v3 - v4


def directnormal_inner(E0, taub, m, ab):
    """Calculates value of Eb in equation (17)

    Equation (17) in ASHRAE Fundamentals 2009 page 14.9

    The equation is
        - Eb = E0 * exp( - taub * power(m, ab) )

    Parameters
    ----------
    E0 : float
        Extraterrestrial normal irradiance [Equation (4)]
    taub : float
        beam optical depth
    m : float
        Air Mass Equation (16)
    ab : float
        beam air mass exponents

    Returns
    -------
    float
        Beam normal irradiance

    """
    # """return the direct normal radiation
    # equation 17
    # Eb = E0 * exp( - taub * power(m, ab) )
    # """
    Eb = E0 * math.exp(-taub * math.pow(m, ab))
    return Eb


def diffhoriz_inner(E0, taud, m, ad):
    """Calculates value of Ed in equation (18)

    Equation (18) in ASHRAE Fundamentals 2009 page 14.9

    The equation is
        - Ed = E0 * exp(-taud * power(m, ad))

    Parameters
    ----------
    E0 : float
        Extraterrestrial normal irradiance [Equation (4)]
    taud : float
        diffuse optical depth
    m : float
        Air Mass Equation (16)
    ad : float
        diffuse air mass exponents

    Returns
    -------
    float
        Diffuse horizontal irradiance

    """
    # """return diffuse horizontal radiation
    # equation 18
    # Ed = E0 * exp(-taud * power(m, ad))"""
    # print math.exp(-taud * math.pow(m, ad))
    Ed = E0 * math.exp(-taud * math.pow(m, ad))
    return Ed


def directnormal(taub, taud, alt, daynum=None, thedate=None):
    """calculate direct normal irradiance from a clear sky

    - calculations for a specific day and specific sun altitude
    - taub and taud can be got from the .stat file
    - the .stat comes with the weather files (.epw, .ddy, .stat)
    - use either daynum or thedate
    - Calls directnormal_inner() that calculates equation (17)

    Parameters
    ----------
    taub : float
        Clear Sky Optical Depth for Beam Irradiance
    taud : float
        Clear Sky Optical Depth for Diffuse Irradiance
    alt : float
        Altitude of the sun
    daynum : int
        daynum is the day of the year
    thedate : datetime.datetime()
        daynum is calculated from the datetime object

    Returns
    -------
    float
        Direct Normal irradiance from a clear sky

    """
    E0 = ETradiation(daynum=daynum, thedate=thedate)
    m = airmass(alt)
    ab = getab(taub, taud)
    Eb = directnormal_inner(E0, taub, m, ab)
    return Eb


def diffusehorizontal(taub, taud, alt, daynum=None, thedate=None):
    """calculate diffuse horizontal irradiance from a clear sky

    - calculations for a specific day and specific sun altitude
    - taub and taud can be got from the .stat file
    - the .stat comes with the weather files (.epw, .ddy, .stat)
    - use either daynum or thedate
    - Calls diffhoriz_inner() that calculates equation (18)

    Parameters
    ----------
    taub : float
        Clear Sky Optical Depth for Beam Irradiance
    taud : float
        Clear Sky Optical Depth for Diffuse Irradiance
    alt : float
        Altitude of the sun
    daynum : int
        daynum is the day of the year
    thedate : datetime.datetime()
        daynum is calculated from the datetime object

    Returns
    -------
    float
        Direct Normal irradiance from a clear sky

    """
    E0 = ETradiation(daynum=daynum, thedate=thedate)
    m = airmass(alt)
    ad = getad(taub, taud)
    # print E0, taud, m, ad
    Ed = diffhoriz_inner(E0, taud, m, ad)
    return Ed



# -*- coding: utf-8 -*-

"""Top-level package for pyclearsky."""

__author__ = """Santosh Philip"""
__email__ = 'santosh@example.com'
# __version__ = '0.3.0'
# src/your_package/__init__.py
from importlib.metadata import version, PackageNotFoundError

try:
    __version__ = version("pyclearsky")   # must match [project] name =
except PackageNotFoundError:
    __version__ = "0.0.0.dev0"                   # fallback for editable installs / dev



"""helpers for pytest"""

# taken from python's unit test
# may be covered by Python's license 

def almostequal(first, second, places=7, printit=True):
    """docstring for almostequal"""
    if round(abs(second-first), places) != 0:
        if printit:
            print round(abs(second-first), places)
            print "notalmost: %s != %s" % (first, second)
        return False
    else:
        return True